<?php

namespace Controllers;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DriverManager;
use Zeedhi\Framework\Session\Session;
######
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\HTTP\Kernel;


class PebbianConnectionFactory extends \Zeedhi\Framework\ORM\ConnectionFactory {

    public static function updateConnectionParams(array $connectionParams, $config = null, $eventManager = null, Session $session = null, Kernel $kernel, $environment){
        $headers = $kernel->getHttpRequest()->getHeaders();
        $requestUri = explode('index.php', trim($_SERVER['REQUEST_URI']));
		$route      = array_pop($requestUri);
		$publicRoutes = json_decode(file_get_contents(__DIR__ . '/../../config/publicRoutes.json'), true);
        $isPublicRoute = in_array($route, $publicRoutes);
        if(!$isPublicRoute){
            $emailOper = $headers->get("Emailoper");
            $senhaOper = $headers->get("Senhaoper");
            $query = "SELECT * FROM OPERADOR
                        WHERE DSEMAILOPER = :DSEMAILOPER
                        AND CDSENHAOPERWEB = :CDSENHAOPERWEB";
            
            $params = array (
                        'DSEMAILOPER' => $emailOper,
                        'CDSENHAOPERWEB' => $senhaOper
                    );
            $connection = DriverManager::getConnection($connectionParams, $config, $eventManager); 
            $resultOperator = $connection->fetchAll($query, $params);
            if(!empty($resultOperator)){
                $resultOperator = $resultOperator[0];
                $nrOrg = $resultOperator['NRORG'];
                $environment->setCdFilial($resultOperator['CDFILIALSELECIONADA']);
                $environment->setNrOrgTrab($resultOperator['NRORG']);
                $environment->setNrOrg($resultOperator['NRORG']);
                $environment->setCdOperador($resultOperator['CDOPERADOR']);
            }
            else{
                $nrOrg = false;
                throw new \Exception('Usuario/senha incorretos');
            }
            if($session) {
                $nrOrg = $nrOrg?? ($session->has('nrorg') ? $session->get('nrorg') : $session->get('NRORG'));
                self::setCurrentDBName($connectionParams, $session);
            }
    
            if ($nrOrg && isset($connectionParams['useVpd']) && $connectionParams['useVpd'] == 'true') {
                $connectionParams = self::factoryVPDConnectionParams($connectionParams, $nrOrg);
            }
    
            if(empty($connectionParams["password"])) {
                $connectionParams["password"] = '';
            } else if(isset($connectionParams["isEncrypted"]) && $connectionParams["isEncrypted"] == 'true') {
                $cryptKey = isset($connectionParams["cryptKey"]) ? $connectionParams["cryptKey"] : null;
                $connectionParams["password"] = Utilities::zeedhiDecrypt($connectionParams["password"], $cryptKey);
                $connectionParams["isEncrypted"] = false;
            }
            
            self::checkOMacl($resultOperator['NRORG'], $route);
        }
        $connection = DriverManager::getConnection($connectionParams, $config, $eventManager);
        self::insertSQLWebTunning($connection, $session);
        
        return $connection;
    }

    protected static function setCurrentDBName($connectionParams, $session) {
        //Set current DBNAME in session for use in login session
        if(isset($connectionParams['sessionMode'])) {
            $encryptedDBName = Utilities::zeedhiEncrypt($connectionParams['dbname']);
            $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '', $encryptedDBName));
        } else {
            $encryptedUser = Utilities::zeedhiEncrypt($connectionParams['user']);
            $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '',$encryptedUser));
        }
    }

    protected static function factoryVPDConnectionParams($connectionParams, $nrOrg) {
        if (isset($connectionParams['vpdWithWallet']) && $connectionParams['vpdWithWallet'] == 'true') {
            $connectionParams['user']        = '/';
            $connectionParams['password']    = '';
            $connectionParams['dbname']      = 'USR_ORG_' . $nrOrg;
            $connectionParams['sessionMode'] = -2147483648;

            unset($connectionParams['host']);
            unset($connectionParams['port']);
        } else {
            $connectionParams['user'] = 'USR_ORG_' . $nrOrg;
            $connectionParams['password'] = $connectionParams['vpdPassword'];
            if (isset($connectionParams['isWallet']) && $connectionParams['isWallet'] == 'true') {
                $connectionParams['dbname']  = $connectionParams['vpdDBNAME'];
                $connectionParams['host']    = $connectionParams['vpdHost'];
                $connectionParams['port']    = $connectionParams['vpdPort'];
                $connectionParams['service'] = $connectionParams['vpdService'];
                unset($connectionParams['sessionMode']);
            }
        }
        return $connectionParams;
    }

    /**
     * @param Connection $connection
     * @param Session $session
     * @throws \Exception
     */
    public static function insertSQLWebTunning($connection, $session) {
        try {
            if(Utilities::getUseSQLWebTunningParameter()) {
                $query = "INSERT INTO SQLWEBTUNNING (DSREMOTEADDR, DSREMOTEHOST, DSREMOTEAPPL, DSSERVERRDM,
                                                     DSSERVERDLL,  DTSTARTDATE,  DSREMOTEOPER, DSSERVMETHOD)
                               VALUES (:DSREMOTEADDR, :DSREMOTEHOST, :DSREMOTEAPPL, :DSSERVERRDM,
                                       :DSSERVERDLL,  SYSDATE,       :DSREMOTEOPER, :DSSERVMETHOD)";

				if(isset($_SERVER['HTTP_HOST'])) {
					$host = $_SERVER['HTTP_HOST'];
				} else if(isset($_SERVER['SERVER_NAME'])) {
					$host = $_SERVER['SERVER_NAME'];
				} else {
					$host = 'EMPTY_HOST';
				}
				if(isset($_SERVER['REQUEST_URI'])) {
					$requestUri = explode('index.php', trim($_SERVER['REQUEST_URI']));
					$route      = array_pop($requestUri);	
				} else {
					$route = 'EMPTY_ROUTE';
				}
				
                $cdoperador = $session && $session->has('CDOPERADOR') ? $session->get('CDOPERADOR') : 'SESSAO_N_INI';
                $hostArr    = explode('.', $host);
                $hostInit   = array_shift($hostArr);
                
                $params = array (
                    'DSREMOTEADDR' => trim(substr(gethostbyname($host), 0, 15)),
                    'DSREMOTEHOST' => trim(substr($hostInit, 0, 15)),
                    'DSREMOTEAPPL' => 'INTEGRACAO',
                    'DSSERVERRDM'  => trim(implode('.', $hostArr)),
                    'DSSERVERDLL'  => 'ZEEDHI_REQUEST',
                    'DSREMOTEOPER' => $cdoperador,
                    'DSSERVMETHOD' => substr($route, 0, 30)
                );
                
                $connection->executeQuery($query, $params);
            }
        } catch(\Exception $e) {
            var_dump($e->getMessage());
        }
    }
    
    public function checkOMacl($nrorg, $route){
        try{
            $omUrl =  "https://om.teknisa.com/backend/index.php";
            $integration = json_decode(file_get_contents(__DIR__ . '/../../vendor/teknisa/libraries-next/config/integration.json'), true);
            $clientSecret = $integration[0]['clientSecret'];
            $clientId = $integration[0]['clientId'];
            $productId = 7195;
            $token = self::getToken($omUrl, $clientSecret, $clientId);
            $requestType = '/api/findProductElement';
            $dataSetResultPos = 'productElement';
            $keepConnected = false;
            $nr_org_id = self::getOrganizationInfo($token, $productId, $clientSecret, $omUrl, $nrorg);
    
            $data = [
                'requestType' => 'FilterData',
                'filter' => [
                    [
                        'name' => 'PRODUCT_ID',
                        'operator' => '=',
                        'value' => $productId
                    ],
                    [
                        'name' => 'NRPRODUTO_ORG_ID',
                        'operator' => '=',
                        'value' => $nr_org_id
                    ],
                    [
                        'name' => 'SHOW_SERVER_INFO',
                        'operator' => '=',
                        'value' => false
                    ]
                ]
            ];
            
            $avaliableScreens = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $clientSecret, $omUrl, $keepConnected);
            
            $route = str_replace('/crudN', '', $route);
            $hasAccess = self::array_some($avaliableScreens, function($row) use($route){
                            return $row['UNIQUE_ID'] == $route;
                        });
                        
            if(!$hasAccess)
                throw new \Exception('Acesso não liberado para uso da rota.');
                
        }catch(\Exception $e) {
            throw new \Exception('Acesso não liberado para uso da rota.');
        }
   }
   
   public function getToken($uri = null, $clientSecret, $clientId){
        $token = null; 
        $requestType = '/lib_requestToken'; 
        $data = array(
            'CLIENT_ID' => $clientId,
            'CLIENT_SECRET' => $clientSecret
        ); 
        $dataSetResultPos = 'token'; 
        $privateKey = $clientSecret; 
        $keepConnected = false;
        $data = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $uri, $keepConnected);
        return $data[0];
   }
   
   public function getOrganizationInfo($token, $productId, $clientSecret, $uri, $nrorg){
       
        $requestType = '/api/findOrganizationInformation'; 
        $data = array(
            'requestType' => 'FilterData',
            'filter' => array(
                array('name' => 'PRODUCT_ID', 'operator' => '=', 'value' => $productId),
                array('name' => 'SHOW_SERVER_INFO', 'operator' => '=', 'value' => false)
            )
        ); 
        $dataSetResultPos = 'organizationInformation'; 
        $keepConnected = false;
        
        $organizationInformation = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $clientSecret, $uri, $keepConnected);
        $organization = current(array_filter($organizationInformation,function($org) use($nrorg){
            return $org['NRORG'] == $nrorg;
        }));
        
        return $organization['NRPRODUTO_ORG_ID']?? [];
   }
   
   public function array_some(array $array, callable $fn) {
        foreach ($array as $value) {
            if($fn($value)) {
                return true;
            }
        }
        return false;
    }


}